# -*- coding: utf-8 -*-
"""
Optimus project for Foundation toast
"""
__version__ = """19.19.0"""
__cookiecutter_context__ = {'_template': '/home/emencia/projects/apps/Pims-dev/cookiecutter-optimus-foundation6', 'directory_name': 'foundation_toast', 'project_name': 'Foundation toast', '_copy_without_render': ['.gitignore', 'boussole.json', 'requirements.txt', 'locale', 'sources']}
